@@ -55,7 +55,7 @@ ARGUMENT_REGEXES = {
 }
 
 
-def get_file_content(markdown, abs_src_path):
+def get_file_content(markdown, abs_src_path, cumulative_heading_offset=0):
     page_src_path = Path(abs_src_path)
 
     def found_include_tag(match):
@@ -100,14 +100,13 @@ def get_file_content(markdown, abs_src_path):
 
         #   string options
         start_match = re.search(ARGUMENT_REGEXES['start'], arguments_string)
-        if start_match is not None:
-            start = process.interpret_escapes(start_match.group(1))
-            _, _, text_to_include = text_to_include.partition(start)
-
         end_match = re.search(ARGUMENT_REGEXES['end'], arguments_string)
-        if end_match is not None:
-            end = process.interpret_escapes(end_match.group(1))
-            text_to_include, _, _ = text_to_include.partition(end)
+
+        text_to_include, _, _ = process.filter_inclusions(
+            start_match,
+            end_match,
+            text_to_include,
+        )
 
         # nested includes
         new_text_to_include = get_file_content(text_to_include, file_path_abs)
@@ -185,16 +184,13 @@ def get_file_content(markdown, abs_src_path):
 
         #   string options
         start_match = re.search(ARGUMENT_REGEXES['start'], arguments_string)
-        start = None
-        if start_match is not None:
-            start = process.interpret_escapes(start_match.group(1))
-            _, _, text_to_include = text_to_include.partition(start)
-
         end_match = re.search(ARGUMENT_REGEXES['end'], arguments_string)
-        end = None
-        if end_match is not None:
-            end = process.interpret_escapes(end_match.group(1))
-            text_to_include, _, _ = text_to_include.partition(end)
+
+        text_to_include, start, end = process.filter_inclusions(
+            start_match,
+            end_match,
+            text_to_include,
+        )
 
         # relative URLs rewriting
         if bool_options['rewrite-relative-urls']['value']:
@@ -217,25 +213,29 @@ def get_file_content(markdown, abs_src_path):
         else:
             text_to_include = _includer_indent + text_to_include
 
-        # nested includes
-        new_text_to_include = get_file_content(text_to_include, file_path_abs)
-        if new_text_to_include != text_to_include:
-            text_to_include = new_text_to_include
-
         # heading offset
+        offset = 0
         offset_match = re.search(
             ARGUMENT_REGEXES['heading-offset'],
             arguments_string,
         )
         if offset_match:
-            offset = int(offset_match.group(1))
-            lines = []
-            for line in text_to_include.splitlines(keepends=True):
-                if line.startswith('#'):
-                    lines.append('#' * offset + line)
-                else:
-                    lines.append(line)
-            text_to_include = ''.join(lines)
+            offset += int(offset_match.group(1))
+
+        # nested includes
+        new_text_to_include = get_file_content(
+            text_to_include,
+            file_path_abs,
+            cumulative_heading_offset=cumulative_heading_offset,
+        )
+        if new_text_to_include != text_to_include:
+            text_to_include = new_text_to_include
+
+        if offset_match:
+            text_to_include = process.increase_headings_offset(
+                text_to_include,
+                offset=offset + cumulative_heading_offset,
+            )
 
         if not bool_options['comments']['value']:
             return text_to_include
